#!/bin/bash

# shell script to build cross tool chain for eCos development

set -e

# warn user if path name contains spaces
if expr index "$PWD" " ">/dev/null; then
    echo " 

You're trying to build in a path that contains spaces.

Path names containing spaces confuse many shell scripts.  Admittedly,
a well-written shell script ought to tolerate spaces in paths and
filenames.  But, tolerating spaces requires extra typing and increases
the script's carbon footprint, so many scripts (including this one)
don't.

Please try again in a location who's path contains no spaces.
"
    exit 1
fi

if expr $(uname) : CYGWIN >/dev/null; then
    echo "fixing cygwin mounts"
    set -x; mount -f -b c:/cygwin/tmp /tmp
    set -x; mount -f -b $(cygpath -m $TMP) $TMP
    set -x; mount -f -b $(cygpath -m $TEMP) $TEMP
    set -x; mount -f -b $(cygpath -m $PWD) $PWD
fi

DSTDIR=$PWD/gnutools
SRCDIR=$PWD/src
CURDIR=$PWD

function apply-patches {
PATCHDIR=$1
for f in $PATCHDIR/*
do
    test -f $f || { echo "no patches: directory $PATCHDIR is not found or is empty"; return 0; }
    echo "applying $f in $PWD"
    patch -p0 < $f
done
}

function build-it {
    WHAT=$1
    VER=$2
    echo "===== unpacking $WHAT ====="
    pushd $SRCDIR
    tar xvf $CURDIR/$WHAT-$VER.tar.*
    echo "===== patching $WHAT ====="
    apply-patches $CURDIR/$WHAT-$VER-patches
    popd
    echo "===== configuring $WHAT ====="
    rm -rf build-$WHAT
    mkdir build-$WHAT
    pushd build-$WHAT
    $SRCDIR/$WHAT-$VER/configure --target=arm-elf --prefix=$DSTDIR
    echo "===== building $WHAT ====="
    make -w all
    echo "===== installing $WHAT ====="
    make -w install
    echo "===== done with $WHAT ====="
    popd
}

function build-gcc {
    VER=$1
    NLBVER=$2
    echo "===== unpacking gcc ====="
    pushd $SRCDIR
    tar xvf $CURDIR/gcc-core-$VER.tar.bz2
    tar xvf $CURDIR/gcc-g++-$VER.tar.bz2
    echo "===== patching gcc ====="
    apply-patches $CURDIR/gcc-$VER-patches
    echo "===== unpacking newlib ====="
    tar xvf $CURDIR/newlib-$NLBVER.tar.gz
    mv newlib-$NLBVER/newlib gcc-$VER
    mv newlib-$NLBVER/libgloss gcc-$VER
    echo "===== patching newlib ====="
    apply-patches $CURDIR/newlib-$NLBVER-patches
    popd
    echo "===== configuring gcc ====="
    PATH=$DSTDIR/bin:$PATH
    rm -rf build-gcc
    mkdir build-gcc
    pushd build-gcc
    $SRCDIR/gcc-$VER/configure --target=arm-elf \
        --prefix=$DSTDIR --enable-languages=c,c++ \
        --with-gnu-as --with-gnu-ld --with-newlib \
        --with-gxx-include-dir=$DSTDIR/arm-elf/include
    echo "===== building gcc ====="
    make -w all
    echo "===== installing gcc ====="
    make -w install
    echo "===== done with gcc ====="
    popd
}

SOURCELIST="
http://ftp.gnu.org/gnu/binutils/binutils-2.19.tar.bz2
http://gcc.releasenotes.org/releases/gcc-3.4.3/gcc-core-3.4.3.tar.bz2
http://gcc.releasenotes.org/releases/gcc-3.4.3/gcc-g++-3.4.3.tar.bz2
ftp://sources.redhat.com/pub/newlib/newlib-1.15.0.tar.gz
ftp://sourceware.org/pub/gdb/old-releases/gdb-6.6.tar.bz2
ftp://sourceware.org/pub/insight/releases/insight-6.6.tar.bz2
"

function download-sources {
    wget -c $SOURCELIST
}


test " $1" = " -d" && download-sources

rm -rf $DSTDIR
mkdir -p $DSTDIR

rm -rf $SRCDIR
mkdir -p $SRCDIR

build-it binutils 2.19
build-gcc 3.4.3 1.15.0
build-it gdb  6.6
build-it insight 6.6
